require File.dirname(__FILE__) + '/../spec_helper'

describe LecturesController, "#route_for" do

  it "should map { :controller => 'lectures', :action => 'index' } to /lectures" do
    route_for(:controller => "lectures", :action => "index").should == "/lectures"
  end
  
  it "should map { :controller => 'lectures', :action => 'new' } to /lectures/new" do
    route_for(:controller => "lectures", :action => "new").should == "/lectures/new"
  end
  
  it "should map { :controller => 'lectures', :action => 'show', :id => 1 } to /lectures/1" do
    route_for(:controller => "lectures", :action => "show", :id => 1).should == "/lectures/1"
  end
  
  it "should map { :controller => 'lectures', :action => 'edit', :id => 1 } to /lectures/1/edit" do
    route_for(:controller => "lectures", :action => "edit", :id => 1).should == "/lectures/1/edit"
  end
  
  it "should map { :controller => 'lectures', :action => 'update', :id => 1} to /lectures/1" do
    route_for(:controller => "lectures", :action => "update", :id => 1).should == "/lectures/1"
  end
  
  it "should map { :controller => 'lectures', :action => 'destroy', :id => 1} to /lectures/1" do
    route_for(:controller => "lectures", :action => "destroy", :id => 1).should == "/lectures/1"
  end
  
end

describe LecturesController, "#params_from" do

  it "should generate params { :controller => 'lectures', action => 'index' } from GET /lectures" do
    params_from(:get, "/lectures").should == {:controller => "lectures", :action => "index"}
  end
  
  it "should generate params { :controller => 'lectures', action => 'new' } from GET /lectures/new" do
    params_from(:get, "/lectures/new").should == {:controller => "lectures", :action => "new"}
  end
  
  it "should generate params { :controller => 'lectures', action => 'create' } from POST /lectures" do
    params_from(:post, "/lectures").should == {:controller => "lectures", :action => "create"}
  end
  
  it "should generate params { :controller => 'lectures', action => 'show', id => '1' } from GET /lectures/1" do
    params_from(:get, "/lectures/1").should == {:controller => "lectures", :action => "show", :id => "1"}
  end
  
  it "should generate params { :controller => 'lectures', action => 'edit', id => '1' } from GET /lectures/1;edit" do
    params_from(:get, "/lectures/1/edit").should == {:controller => "lectures", :action => "edit", :id => "1"}
  end
  
  it "should generate params { :controller => 'lectures', action => 'update', id => '1' } from PUT /lectures/1" do
    params_from(:put, "/lectures/1").should == {:controller => "lectures", :action => "update", :id => "1"}
  end
  
  it "should generate params { :controller => 'lectures', action => 'destroy', id => '1' } from DELETE /lectures/1" do
    params_from(:delete, "/lectures/1").should == {:controller => "lectures", :action => "destroy", :id => "1"}
  end
  
end

describe LecturesController, "handling GET /lectures" do

  before do
    @lecture = mock_model(Lecture)
    Lecture.stub!(:find).and_return([@lecture])
  end
  
  def do_get
    get :index
  end
  
  it "should be successful" do
    do_get
    response.should be_success
  end

  it "should render index template" do
    do_get
    response.should render_template('index')
  end
  
  it "should find all lectures" do
Lecture.should_receive(:find).with(:all, :order => :start).and_return([@lecture])
    do_get
  end
  
  it "should assign the found lectures for the view" do
    do_get
    assigns[:lectures].should == [@lecture]
  end
end

describe LecturesController, "handling GET /lectures/1" do

  before do
    @lecture = mock_model(Lecture)
    Lecture.stub!(:find).and_return(@lecture)
  end
  
  def do_get
    get :show, :id => "1"
  end

  it "should be successful" do
    do_get
    response.should be_success
  end
  
  it "should render show template" do
    do_get
    response.should render_template('show')
  end
  
  it "should find the lecture requested" do
    Lecture.should_receive(:find).with("1").and_return(@lecture)
    do_get
  end
  
  it "should assign the found lecture for the view" do
    do_get
    assigns[:lecture].should equal(@lecture)
  end
end

describe LecturesController, "handling GET /lectures/new" do

  before do
    @lecture = mock_model(Lecture)
    Lecture.stub!(:new).and_return(@lecture)
  end
  
  def do_get
    get :new
  end

  it "should be successful" do
    do_get
    response.should be_success
  end
  
  it "should render new template" do
    do_get
    response.should render_template('new')
  end
  
  it "should create an new lecture" do
    Lecture.should_receive(:new).and_return(@lecture)
    do_get
  end
  
  it "should not save the new lecture" do
    @lecture.should_not_receive(:save)
    do_get
  end
  
  it "should assign the new lecture for the view" do
    do_get
    assigns[:lecture].should equal(@lecture)
  end
end

describe LecturesController, "handling GET /lectures/1/edit" do

  before do
    @lecture = mock_model(Lecture)
    Lecture.stub!(:find).and_return(@lecture)
  end
  
  def do_get
    get :edit, :id => "1"
  end

  it "should be successful" do
    do_get
    response.should be_success
  end
  
  it "should render edit template" do
    do_get
    response.should render_template('edit')
  end
  
  it "should find the lecture requested" do
    Lecture.should_receive(:find).and_return(@lecture)
    do_get
  end
  
  it "should assign the found Lecture for the view" do
    do_get
    assigns[:lecture].should equal(@lecture)
  end
end

describe LecturesController, "handling POST /lectures" do

  before do
    @lecture = mock_model(Lecture, :to_param => "1")
    Lecture.stub!(:new).and_return(@lecture)
  end
  
  def post_with_successful_save
    @lecture.should_receive(:save).and_return(true)
    post :create, :lecture => {}
  end
  
  def post_with_failed_save
    @lecture.should_receive(:save).and_return(false)
    post :create, :lecture => {}
  end
  
  it "should create a new lecture" do
    Lecture.should_receive(:new).with({}).and_return(@lecture)
    post_with_successful_save
  end

  it "should redirect to the new lecture on successful save" do
    post_with_successful_save
    response.should redirect_to(lecture_url("1"))
  end

  it "should re-render 'new' on failed save" do
    post_with_failed_save
    response.should render_template('new')
  end
end

describe LecturesController, "handling PUT /lectures/1" do

  before do
    @lecture = mock_model(Lecture, :to_param => "1")
    Lecture.stub!(:find).and_return(@lecture)
  end
  
  def put_with_successful_update
    @lecture.should_receive(:update_attributes).and_return(true)
    put :update, :id => "1"
  end
  
  def put_with_failed_update
    @lecture.should_receive(:update_attributes).and_return(false)
    put :update, :id => "1"
  end
  
  it "should find the lecture requested" do
    Lecture.should_receive(:find).with("1").and_return(@lecture)
    put_with_successful_update
  end

  it "should update the found lecture" do
    put_with_successful_update
    assigns(:lecture).should equal(@lecture)
  end

  it "should assign the found lecture for the view" do
    put_with_successful_update
    assigns(:lecture).should equal(@lecture)
  end

  it "should redirect to the lecture on successful update" do
    put_with_successful_update
    response.should redirect_to(lecture_url("1"))
  end

  it "should re-render 'edit' on failed update" do
    put_with_failed_update
    response.should render_template('edit')
  end
end

describe LecturesController, "handling DELETE /lectures/1" do

  before do
    @lecture = mock_model(Lecture, :destroy => true)
    Lecture.stub!(:find).and_return(@lecture)
  end
  
  def do_delete
    delete :destroy, :id => "1"
  end

  it "should find the lecture requested" do
    Lecture.should_receive(:find).with("1").and_return(@lecture)
    do_delete
  end
  
  it "should call destroy on the found lecture" do
    @lecture.should_receive(:destroy)
    do_delete
  end
  
  it "should redirect to the lectures list" do
    do_delete
    response.should redirect_to(lectures_url)
  end
end

describe LecturesController, "handling PUT /lectures/vote_for/1?vote=42" do
  
  before(:each) do
    @lecture = mock_model(Lecture, :null_object => true)
    Lecture.stub!(:find).and_return(@lecture)
  end
  
  it "should find lecture by id" do
    Lecture.should_receive(:find).and_return(@lecture)
    put :vote_for, :id => @lecture.id, :vote => "42"
  end
  
  it "should update the lecture voting" do
    @lecture.should_receive(:account_vote).with(42)
    @lecture.should_receive(:update).and_return(true)
    put :vote_for, :id => @lecture.id, :vote => "42"
  end
  
  it "should redirect to lectures list" do
    put :vote_for, :id => @lecture.id, :vote => "42"
    response.should redirect_to(lectures_url)
  end
end