require File.dirname(__FILE__) + '/../../spec_helper'

describe "/lectures/index.rhtml" do
  include LecturesHelper
  
  before do
    lecture_98 = mock_model(Lecture, :null_object => true)
    lecture_98.stub!(:title).and_return("Title")
    lecture_98.stub!(:speakers).and_return("MyString")
    lecture_98.stub!(:start).and_return(Time.parse("14:00"))
    lecture_98.stub!(:end).and_return(Time.parse("15:00"))
    lecture_99 = mock_model(Lecture, :null_object => true)
    lecture_99.stub!(:title).and_return("Title")
    lecture_99.stub!(:speakers).and_return("MyString")
    lecture_99.stub!(:start).and_return(Time.parse("15:00"))
    lecture_99.stub!(:end).and_return(Time.parse("16:00"))

    assigns[:lectures] = @lectures = [lecture_98, lecture_99]
  end

  it "should render logo" do
    render "/lectures/index.rhtml"
    response.should have_tag("img#logo")
  end

  it "should render list of lectures" do
    render "/lectures/index.rhtml"
    response.should have_tag("tr>td", "Title", 2)
    response.should have_tag("tr>td", "MyString", 2)
  end

	it "should format list" do
	  render "/lectures/index.rhtml"
	  response.should have_tag("table#list")
	end
	
  it "should format start and end date" do
    render "/lectures/index.rhtml"
    response.should have_tag("tr>td", "14:00-15:00")
    response.should have_tag("tr>td", "15:00-16:00")
  end

  it "should format column headers" do
    render "/lectures/index.rhtml"
    response.should have_tag("tr>th", "Título")
    response.should have_tag("tr>th", "Palestrantes")
    response.should have_tag("tr>th", "Horário")
    response.should have_tag("tr>th", "Votos")
  end

  it "should remove links" do
    render "/lectures/index.rhtml"    
    response.should_not have_tag("a", "Show")
    response.should_not have_tag("a", "Edit")
    response.should_not have_tag("a", "Destroy")
  end
  
  it "should edit details on lecture title" do
    render "/lectures/index.rhtml"    
    response.should have_tag("a", "Title", 2)
  end
  
  it "should render partial" do
    template.expect_render(:partial => 'lecture', :collection => @lectures)
    render "/lectures/index.rhtml"
  end
  
  it "should display feedback buttons with links" do
    render "/lectures/index.rhtml"
    response.should have_tag("tr>td>a>img", 4)
  end
  
  it "should highlight lecture" do
    template.should_receive(:highlight).twice.and_return("")
    render "/lectures/index.rhtml"
  end
end

